#  Copyright 2022 The Matrix.org Foundation C.I.C.
#
#  Licensed under the Apache License, Version 2.0 (the "License");
#  you may not use this file except in compliance with the License.
#  You may obtain a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
#  Unless required by applicable law or agreed to in writing, software
#  distributed under the License is distributed on an "AS IS" BASIS,
#  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#  See the License for the specific language governing permissions and
#  limitations under the License.
from unittest import TestCase

from matrix_common.versionstring import get_distribution_version_string


class TestVersionString(TestCase):
    def test_our_own_version_string(self) -> None:
        """Sanity check that we get the version string for our own package.

        Check that it's a nonempty string.
        """
        version = get_distribution_version_string("matrix-common")
        self.assertIsInstance(version, str)
        self.assertTrue(version)
